from enum import Enum
from typing import Tuple, Optional, Callable, Any,List


        
def Parameters():
    '''
    Annotation to place before a class method.
    This annotation indicates that the method takes no parameters
    and that it returns list of lists. Every inner list
    is to be used to call the constructor, so the number
    of arguments must match the __init__ function parameters.
    All it does is setting an attribute to the method
    
    '''    

    
    def doAnnotate(annotatedmethod):
        #337 workaround @staticmethod "bug"
        if isinstance(annotatedmethod, staticmethod):
            annotatedmethod = annotatedmethod.__func__
        setattr(annotatedmethod,'__parameters__', True)
        return annotatedmethod

    
    '''
    A bit complex mechanism that just sets
    attribute __parameters__ of the annotated class
    '''
    return doAnnotate


##### support functions.
# these are static functions because they handle class objects directly


 
def getParameters( ob )-> Optional[Callable[..., List[List[Any]]]]:
    '''
    @param ob: a class that may have a @Before annotated method
    @return method in clas that has @Before annotation, or None
    '''
    for fname in dir(ob):
        func = getattr(ob, fname)
        # 397 check exact value of attribute, avoiding mocks
        if hasattr(func, '__parameters__') and getattr(func, '__parameters__')==True:
            return func
    return None


