from __future__ import annotations
from abc import abstractmethod
from typing import Type, Callable,List
from unitpy.Before import getBefore
from unitpy.After import getAfter
from unitpy.Parameters import getParameters
from unitpy.RunWith import getRunWith
from unitpy.Test import getTestNames, getExpected
import logging

class Runner:
    
    def __init__(self, clazz:Type):
        '''
        @param clazz the class to run the tests of
        '''
        self._clazz:Type= clazz
        
    @abstractmethod
    def run(self):
        '''
        runs all tests, using the runner specified with the clazz.
        To  be implemented by  actual runners.
        '''
        raise NotImplemented()
    
        
    
        
    def _runall(self, args):
        '''
        run all methods annotated with @Test in clazz.
        @param args a list of arguments for the clazz constructor
        '''
        for method in getTestNames(self._clazz):
            instanz = self._clazz(*args)
            self.before(instanz)
            
            m:Callable=getattr(instanz,method)
            expected= getExpected(m)
            if expected:
                try:
                    m()
                    raise AssertionError(method +"() did not raise  "+str(expected))
                except BaseException as e:
                    if not isinstance(e, expected):
                        raise AssertionError(method + "() was expected to raise "+str(expected)+ " but actually got "+str(e))

            else:
               m()

            self.after(instanz)

    def before(self, ob):
        '''
        @param ob the clazz instance to call @Before method (if available)
        '''
        b:Callable = getBefore(ob)
        if b:
            b()

    def after(self, ob):
        '''
        @param ob the clazz instance to call @Before method (if available)
        '''
        a:Callable = getAfter(self._clazz)
        if a:
            a()
        # HACK we really should close all python herer but we can't 
        # Closing the logging system is most urgent now #423
        # Maybe we need to close more here? 
        logging.shutdown()
        logging.Logger.manager = logging.Manager(logging.Logger.root)

        
def runTests(clazz: Type):
    '''
    runs all tests in a class, using the specified runner.
    '''
    getRunWith(clazz)(clazz).run()


class JUnit4ClassRunner(Runner):
    '''
    runs all methods annotated @Test in the given class
    '''
    def run(self):
        self._runall([])

    
class Parameterized(Runner):
    '''
    runs all methods annotated @Test in the given class
    '''
    def run(self):
        params:List[List] = getParameters(self._clazz)
        if not params:
            raise ValueError("@Parameterized test must have method with @Parameters annotation")
        for param in params():
            self._runall(param)
        
