from enum import Enum
from typing import Tuple, Optional, Callable,List, Optional, Type
from unitpy.Ignore import getIgnored


        
def Test(expected:Optional[Type]=None):
    '''
    Annotation to place before a class method.
    The annotation indicates that this method is a test method.
    All it does is setting an attribute to the method
    
    Argument is the exception class
    
    '''    

    
    def doAnnotate(annotatedmethod):
        setattr(annotatedmethod,'__test__', expected)
        return annotatedmethod

    
    '''
    A bit complex mechanism that just sets
    attribute __test__ of the annotated class
    '''
    return doAnnotate


##### support functions.
# these are static functions because they handle class objects directly



def getTestNames( ob )-> List[str]:
    '''
    @param ob: a class that may have @Test annotated methods
    @return all method names in the class that are annotated @Test.
    This does not return methods because the names will be used
    on a fresh instance of the test class.
    '''
    return [name for name in dir(ob) 
             if hasattr(getattr(ob, name),'__test__') 
             and not getIgnored(getattr(ob, name),) ]
    

def getExpected(method) -> Optional[Type[Exception]]:
    '''
    @param method a method in a class
    @return the exception expected by this method
    '''
    if  hasattr(method, '__test__'):
        return getattr(method, '__test__')
    return None
